#include common_scripts\utility;
#include maps\mp\_utility;
#include maps\mp\perks\_perks;
#include maps\mp\gametypes\_hud_util;
#include maps\mp\mods\disableClassMenu;
#include maps\mp\mods\helpingHints;

// Rallypoints should be destroyed on leaving your team/getting killed
// Doesn't seem to be setting angle on spawn so that you are facing your rallypoint

/*
	Hide and Seek
	Attackers objective: Kill all enemies.
	Defenders objective: Hide as an entity on the map, optionally meleeing attackers.
	Round ends:	When one team is eliminated, or roundlength time is reached
	Map ends:	When one team reaches the score limit, or time limit or round limit is reached
	Respawning:	Players remain dead for the round and will respawn at the beginning of the next round

	For this gametype, special key bindings need to be enabled.

	- Switching into a prop(menu)
	- Switching into a prop(quick)
	- Rotating the prop(fast)
	- Rotating the prop(slow)

	Level requirements
	------------------
		Allied Spawnpoints:
			classname		mp_sd_spawn_attacker
			Allied players spawn from these. Place at least 16 of these relatively close together.

		Axis Spawnpoints:
			classname		mp_sd_spawn_defender
			Axis players spawn from these. Place at least 16 of these relatively close together.

		Spectator Spawnpoints:
			classname		mp_global_intermission
			Spectators spawn from these and intermission is viewed from these positions.
			Atleast one is required, any more and they are randomly chosen between.

	Level script requirements
	-------------------------
		Team Definitions:
			game["seekers"] = "allies";
			game["hiders"] = "axis";

*/

/*QUAKED mp_sd_spawn_attacker(0.0 1.0 0.0) (-16 -16 0) (16 16 72)
Attacking players spawn randomly at one of these positions at the beginning of a round.*/

/*QUAKED mp_sd_spawn_defender(1.0 0.0 0.0) (-16 -16 0) (16 16 72)
Defending players spawn randomly at one of these positions at the beginning of a round.*/

main()
{
	// Make gametype display properly in server browser
	setDvar("ui_gametype", "Hide & Seek");

	// We don't run this gametype in the menu
	if (getdvar("mapname") == "mp_background")
		return;

	// Define allowed seeker weapons
	level.seekerWeapons = [];
	level.seekerWeapons[0] = "m16";
	level.seekerWeapons[1] = "m4";
	level.seekerWeapons[2] = "ump45";
	level.seekerWeapons[3] = "fal";
	level.seekerWeapons[4] = "scar";
	level.seekerWeapons[5] = "ak47";
	level.seekerWeapons[6] = "p90";
	level.seekerWeapons[7] = "famas";

	// Make DVars available
	setDvarIfUninitialized("scr_" + level.gameType + "_roundswitch", 1);
	setDvarIfUninitialized("scr_" + level.gameType + "_timelimit", 5);
	setDvarIfUninitialized("scr_" + level.gameType + "_scorelimit", 1);
	setDvarIfUninitialized("scr_" + level.gameType + "_roundlimit", 0);
	setDvarIfUninitialized("scr_" + level.gameType + "_winlimit", 4);
	setDvarIfUninitialized("scr_" + level.gameType + "_halftime", 0);
	setDvarIfUninitialized("scr_" + level.gameType + "_numlives", 1);

	maps\mp\gametypes\_globallogic::init();
	maps\mp\gametypes\_callbacksetup::SetupCallbacks();
	maps\mp\gametypes\_globallogic::SetupCallbacks();

	level.teamBased = true;

	// General game logic DVars
	registerRoundSwitchDvar(level.gameType, 1, 0, 9);
	registerTimeLimitDvar(level.gameType, 5, 0, 1440);
	registerScoreLimitDvar(level.gameType, 1, 0, 500);
	registerRoundLimitDvar(level.gameType, 0, 0, 12);
	registerWinLimitDvar(level.gameType, 4, 0, 12);
	registerHalfTimeDvar(level.gameType, 0, 0, 1);
	registerNumLivesDvar(level.gameType, 1, 0, 10);

	// Specific DVars for this gametype
	registerPrereleaseDvar(level.gameType, 30, 0, 300);

	// Don't show perks
	SetDvar("scr_showperksonspawn", 0);

	// Don't allow class change
	SetDvar("ui_allow_classchange", 0);

	// event handlers
	//level.onXPEvent = ::onXPEvent;

	level.getSpawnPoint = ::getSpawnPoint;
	level.onSpawnPlayer = ::onSpawnPlayer;
	level.onRespawnDelay = maps\mp\gametypes\_globallogic::blank;

	level.onTimeLimit = ::onTimeLimit;
	level.onHalfTime = maps\mp\gametypes\_gamelogic::default_onHalfTime;
	level.onDeadEvent = ::onDeadEvent;
	level.onOneLeftEvent = ::onOneLeftEvent;

	level.onPrecacheGametype = ::onPrecacheGametype;
	level.onStartGameType = ::onStartGameType;
	level.onPlayerKilled = ::onPlayerKilled;
	level.onNormalDeath = ::onNormalDeath;

	game["dialog"]["gametype"] = "hns";

	if (getDvarInt("g_hardcore"))
		game["dialog"]["gametype"] = "hc_" + game["dialog"]["gametype"];
	else if (getDvarInt("camera_thirdPerson"))
		game["dialog"]["gametype"] = "thirdp_" + game["dialog"]["gametype"];
	else if (getDvarInt("scr_diehard"))
		game["dialog"]["gametype"] = "dh_" + game["dialog"]["gametype"];
	else if (getDvarInt("scr_" + level.gameType + "_promode"))
		game["dialog"]["gametype"] = game["dialog"]["gametype"] + "_pro";

	// stretch regen time
	maps\mp\gametypes\_tweakables::registerTweakable("player", "healthregentime", "scr_player_healthregentime", 10);

	// disable all unwanted kinds of weapons
	maps\mp\gametypes\_tweakables::setTweakableValue("weapon", "allowfrag", "scr_weapon_allowfrags", 0);
	maps\mp\gametypes\_tweakables::setTweakableValue("weapon", "allowsmoke", "scr_weapon_allowsmoke", 0);
	maps\mp\gametypes\_tweakables::setTweakableValue("weapon", "allowflash", "scr_weapon_allowflash", 0);
	maps\mp\gametypes\_tweakables::setTweakableValue("weapon", "allowc4", "scr_weapon_allowc4", 0);
	maps\mp\gametypes\_tweakables::setTweakableValue("weapon", "allowclaymores", "scr_weapon_allowclaymores", 0);
	maps\mp\gametypes\_tweakables::setTweakableValue("weapon", "allowrpgs", "scr_weapon_allowrpgs", 0);
	maps\mp\gametypes\_tweakables::setTweakableValue("weapon", "allowmines", "scr_weapon_allowmines", 0);

	// disable all hardpoints
	maps\mp\gametypes\_tweakables::setTweakableValue("hardpoint", "allowartillery", "scr_hardpoint_allowartillery", 0);
	maps\mp\gametypes\_tweakables::setTweakableValue("hardpoint", "allowuav", "scr_hardpoint_allowuav", 0);
	maps\mp\gametypes\_tweakables::setTweakableValue("hardpoint", "allowsupply", "scr_hardpoint_allowsupply", 0);
	maps\mp\gametypes\_tweakables::setTweakableValue("hardpoint", "allowhelicopter", "scr_hardpoint_allowhelicopter", 0);

	level disableClassMenu();

	initGameFlags();
	gameFlagInit("seekers_released", false);
}

onPrecacheGametype()
{
	logPrint("onPrecacheGametype called");

	game["seekers_released_sound"] = "mp_war_objective_taken";
	game["seekers_blind_sound"] = "elev_run_end";
	game["hiders_changed_sound"] = undefined;

	game["hiders_model_icon"] = "remotemissile_target_friendly";

	game["fx_confetti"] = LoadFX("props/cash_player_drop");
	game["fx_boom"] = LoadFX("props/electricbox4_explode");

	game["menu_models"] = "hns_models";

	PreCacheString(&"HIDEANDSEEK_TIMER_SEEKER_RELEASE");
	PreCacheString(&"HIDEANDSEEK_SEEKERS_RELEASED");
	PreCacheString(&"OBJECTIVES_HIDEANDSEEK_DEFENDER");
	PreCacheString(&"HIDEANDSEEK_HINT_HIDER_DISGUISE");
	PreCacheString(&"HIDEANDSEEK_HINT_HIDER_ANGLELOCK");
	PreCacheString(&"HIDEANDSEEK_HINT_HIDER_THIRDPERSON");
	PreCacheString(&"HIDEANDSEEK_HINT_SEEKER_SEMTEX");

	PreCacheShader(game["hiders_model_icon"]);

	PreCacheMenu(game["menu_models"]);
}

registerPrereleaseDvar(dvarString, defaultValue, minValue, maxValue)
{
	logPrint("registerPrereleaseDvar called");

	registerWatchDvarInt("prereleasetime", defaultValue);

	dvarString = "scr_" + dvarString + "_prereleasetime";

	level.preReleaseTimeDvar = dvarString;
	level.preReleaseTimeMin = minValue;
	level.preReleaseTimeMax = maxValue;
	level.preReleaseTime = getDvarInt(dvarString, defaultValue);

	if (level.preReleaseTime < minValue)
	{
		level.preReleaseTime = minValue;
	}
	else if (level.preReleaseTime > maxValue)
	{
		level.preReleaseTime = maxValue;
	}
}

updateGametypeDvars()
{
	logPrint("updateGametypeDvars called");

	// no time <= {prereleasetime} <= 5 minutes
	level.preReleaseTime = dvarIntValue("prereleasetime", 30, 0, 5 * 60);
}

onStartGameType()
{
	logPrint("onStartGameType called");

	// reset flags
	gameFlagClear("seekers_released");

	if (!IsDefined(game["switchedsides"]))
		game["switchedsides"] = false;

	// Define who's seekers and who's hiders
	if (!IsDefined(game["seekers"]) || !IsDefined(game["hiders"]))
	{
		// compatibility with the game's own game logic
		game["seekers"] = game["attackers"];
		game["hiders"] = game["defenders"];
	}

	// handle side switching
	if (game["switchedsides"])
	{
		oldSeekers = game["seekers"];
		oldHiders = game["hiders"];
		game["seekers"] = oldHiders;
		game["hiders"] = oldSeekers;
		game["attackers"] = game["seekers"];
		game["defenders"] = game["hiders"];
	}

	// team setup
	setDvar("g_TeamName_" + game["hiders"], "Hiders");
	setDvar("g_TeamName_" + game["seekers"], "Seekers");

	// define models that can be changed into for hns_models menu
	for (i = 0; i < 64; i++)
	{
		dvarName = "ui_hns_model_" + i;
		value = "";
		if (i < level.ModelList.size)
		{
			value = level.ModelList[i].RName;
		}
		setDvar(dvarName, value);
	}

	// prevent spectators ghosting through nickchanging
	setClientNameMode("auto_change");

	// objective text
	// TODO
	setObjectiveText(game["seekers"], &"OBJECTIVES_WAR");
	setObjectiveText(game["hiders"], &"OBJECTIVES_HIDEANDSEEK_DEFENDER");
	if (!level.splitscreen)
	{
		setObjectiveScoreText(game["seekers"], &"OBJECTIVES_WAR_SCORE");
		setObjectiveScoreText(game["hiders"], &"OBJECTIVES_SD_DEFENDER_SCORE");
	}
	setObjectiveHintText(game["seekers"], &"OBJECTIVES_WAR_HINT");
	setObjectiveHintText(game["hiders"], &"OBJECTIVES_HIDEANDSEEK_DEFENDER");

	// spawn points
	level.spawnMins = (0, 0, 0);
	level.spawnMaxs = (0, 0, 0);
	maps\mp\gametypes\_spawnlogic::placeSpawnPoints("mp_sd_spawn_attacker");
	maps\mp\gametypes\_spawnlogic::placeSpawnPoints("mp_sd_spawn_defender");

	// center of map
	level.mapCenter = maps\mp\gametypes\_spawnlogic::findBoxCenter( level.spawnMins, level.spawnMaxs );
	setMapCenter(level.mapCenter);

	// game objects on map
	allowed[0] = "dm";
	allowed[1] = "war";
	allowed[2] = level.gameType;
	maps\mp\gametypes\_gameobjects::main(allowed);
	level maps\mp\mods\_modellist::initModelList();

	// scoring
	maps\mp\gametypes\_rank::registerScoreInfo("win", 2);
	maps\mp\gametypes\_rank::registerScoreInfo("loss", 1);
	maps\mp\gametypes\_rank::registerScoreInfo("tie", 1.5);
	//
	maps\mp\gametypes\_rank::registerScoreInfo("kill", 50);
	maps\mp\gametypes\_rank::registerScoreInfo("headshot", 50);
	maps\mp\gametypes\_rank::registerScoreInfo("assist", 20);

	level updateGametypeDvars();

	level thread levelShowModelsOnMinimap();
	level thread levelReleaseTimer();
}


levelShowModelsOnMinimap()
{
	// TODO: Use level-global ID!
	for (i = 0; i < level.EntityList.size; i++)
	{
		ent = level.EntityList[i];
		objID = maps\mp\gametypes\_gameobjects::getNextObjID();
		Objective_Add(objID, "active");
		Objective_OnEntity(objID, ent);
		Objective_Icon(objID, game["hiders_model_icon"]);
		Objective_Team(objID, game["hiders"]);
	}
}

getSpawnPoint()
{
	logPrint("getSpawnPoint called");

	if (self.pers["team"] == game["seekers"])
		spawnPointName = "mp_sd_spawn_attacker";
	else
		spawnPointName = "mp_sd_spawn_defender";

	spawnPoints = maps\mp\gametypes\_spawnlogic::getSpawnpointArray(spawnPointName);
	assert(spawnPoints.size);
	spawnpoint = maps\mp\gametypes\_spawnlogic::getSpawnpoint_Random(spawnPoints);

	return spawnpoint;
}

onSpawnPlayer()
{
	logPrint("onSpawnPlayer called");

	player = self;

	//player.spawnprotection = 0;
	//player thread DoSpawnProtection();

	//player SetClientDvar("cg_scoreboardItemHeight", 13);
	player SetClientDvar("lowAmmoWarningColor1", 0, 0, 0, 0);
	player SetClientDvar("lowAmmoWarningColor2", 0, 0, 0, 0);
	player SetClientDvar("lowAmmoWarningNoAmmoColor1", 0, 0, 0, 0);
	player SetClientDvar("lowAmmoWarningNoAmmoColor2", 0, 0, 0, 0);
	player SetClientDvar("lowAmmoWarningNoReloadColor1", 0, 0, 0, 0);
	player SetClientDvar("lowAmmoWarningNoReloadColor2", 0, 0, 0, 0);

	if (player.pers["team"] == game["hiders"])
	{
		player thread handleHider();
	}
	else if (player.pers["team"] == game["seekers"])
	{
		player thread handleSeeker();
	}

	level notify("spawned_player");
}

onPlayerKilled(eInflictor, attacker, iDamage, sMeansOfDeath, sWeapon, vDir, sHitLoc, psOffsetTime, deathAnimDuration, killId)
{
	logPrint("onPlayerKilled called");

	player = self;

	// If victim was disguised, reset him
	player deleteDisguiseModel();

	// Close menu, intended for hns_models menu
	player closePopupMenu();

	// Update spectator permissions
	thread checkAllowSpectating();
}


checkAllowSpectating()
{
	logPrint("checkAllowSpectating called");

	wait(0.05);

	if (!level.aliveCount[game["seekers"]])
	{
		level.spectateOverride[game["seekers"]].allowEnemySpectate = 1;
		maps\mp\gametypes\_spectating::updateSpectateSettings();
	}

	if (!level.aliveCount[game["hiders"]])
	{
		level.spectateOverride[game["hiders"]].allowEnemySpectate = 1;
		maps\mp\gametypes\_spectating::updateSpectateSettings();
	}
}


onDeadEvent(team)
{
	logPrint("onDeadEvent called");

	if (team == "all")
	{
		thread maps\mp\gametypes\_gamelogic::endGame("tie", game["strings"]["tie"]);
	}
	else if (team == game["seekers"])
	{
		level thread maps\mp\gametypes\_gamelogic::endGame(game["hiders"], game["strings"][game["seekers"]+"_eliminated"]);
	}
	else if (team == game["hiders"])
	{
		level thread maps\mp\gametypes\_gamelogic::endGame(game["seekers"], game["strings"][game["hiders"]+"_eliminated"]);
	}
}


onTimeLimit()
{
	logPrint("onTimeLimit called");

	// Hiders win when time limit reached
	thread maps\mp\gametypes\_gamelogic::endGame(game["hiders"], game["strings"]["time_limit_reached"]);
}


onNormalDeath(victim, attacker, lifeId)
{
	logPrint("onNormalDeath called");

	// Make sure the kill score is set properly
	score = maps\mp\gametypes\_rank::getScoreInfoValue("kill");
	assert(isDefined(score));

	thread maps\mp\_matchdata::logKillEvent(lifeId, victim.pers["team"]);

	// Is this the final kill?
	if (game["state"] == "postgame")
		attacker.finalKill = true;
}


onOneLeftEvent(team)
{
	logPrint("onOneLeftEvent called");

	player = getLastLivingPlayer(team);

	player thread giveLastOnTeamWarning();
}


giveLastOnTeamWarning()
{
	logPrint("giveLastOnTeamWarning called");

	player = self;

	player endon("death");
	player endon("disconnect");
	level endon("game_ended");

	player waitTillRecoveredHealth(3);

	otherTeam = getOtherTeam(player.pers["team"]);
	level thread teamPlayerCardSplash("callout_lastteammemberalive", player, player.pers["team"]);
	level thread teamPlayerCardSplash("callout_lastenemyalive", player, otherTeam);
	level notify("last_alive", player);
}

levelReleaseTimer()
{
	logPrint("levelReleaseTimer called");

	gameFlagClear("seekers_prerelease_setup");
	gameFlagClear("seekers_released");
	gameFlagWait("prematch_done");

	level.preReleaseStart = getTime();

	gameFlagSet("seekers_prerelease_setup");

	if (!level.preReleaseTime)
	{
		gameFlagSet("seekers_released");
		return;
	}

	// Stop the main game timer
	level.timeLimitOverride = true;
	setGameEndTime(0);
	maps\mp\gametypes\_gamelogic::pauseTimer();

	timerDisplays = [];

	// Pre-release timer for hiders now handled in handleHider function!

	// Display a pre-release timer in the center for the seekers
	timerDisplayText = createServerFontString("objective", 1.5, game["seekers"]);
	timerDisplayText setPoint("CENTER", "CENTER", 0, -40);
	timerDisplayText.sort = 1001;
	timerDisplayText setText(&"HIDEANDSEEK_TIMER_SEEKER_RELEASE");
	timerDisplayText.foreground = false;
	timerDisplayText.hideWhenInMenu = true;
	timerDisplays[timerDisplays.size] = timerDisplayText;
	timerDisplay = createServerTimer("hudbig", 1, game["seekers"]);
	timerDisplay setPoint("CENTER", "CENTER", 0, 0);
	timerDisplay.sort = 1001;
	timerDisplay.color = (1,1,0);
	timerDisplay.foreground = false;
	timerDisplay.hideWhenInMenu = true;
	timerDisplay maps\mp\gametypes\_hud::fontPulseInit();
	//timerDisplay setTimer(level.preReleaseTime);
	timerDisplays[timerDisplays.size] = timerDisplay;

	waittillframeend; // wait till cleanup of previous start timer if multiple happen at once
	timerSeconds = level.preReleaseTime;
	for (timerSeconds = level.preReleaseTime; timerSeconds > 0 && !level.gameEnded; timerSeconds--)
	{
		timerDisplays[timerDisplays.size - 1] thread maps\mp\gametypes\_hud::fontPulse(level);
		wait (timerDisplays[timerDisplays.size - 1].inFrames * 0.05);
		timerDisplays[timerDisplays.size - 1] SetValue(timerSeconds);
		wait (1 - (timerDisplays[timerDisplays.size - 1].inFrames * 0.05));
	}

	if (!level.gameEnded)
	{
		// Prerelease phase done
		gameFlagSet("seekers_released");

		// Resume the main game timer
		setGameEndTime(getTime() + (getTimeLimit() * 60 * 1000));
		level.timeLimitOverride = false;
		maps\mp\gametypes\_gamelogic::resumeTimer();
	}

	for (index = 0; index < timerDisplays.size; index++)
		timerDisplays[index] destroyElem();
}

handleSeeker() {
	logPrint("handleSeeker called");

	player = self;

	player endon("disconnect");
	player endon("death");
	level endon("game_ended");

	// Fix when team switching for lower message
	player clearLowerMessage("seekerreleasemsg");

	// Wait for _playerlogic to give the player his default weapons and perks
	player waittill("spawned_player");

	// Disable third-person mode
	player SetClientDvar("cg_thirdperson", 0);

	// Take weapons and perks
	player ClearPerks();
	player TakeAllWeapons();

	waittillframeend;

	gameFlagWait("prematch_done");

	if (!gameFlag("seekers_released")) {
		// Blind player if seekers weren't released yet
		player playLocalSound(game["seekers_blind_sound"]);
		player VisionSetNakedForPlayer("blacktest", 0.1);

		// Freeze, hide
		player FreezeControls(true);
		player Hide();

		gameFlagWait("seekers_released");
		player playLocalSound(game["seekers_released_sound"]);

		// Unfreeze, show
		player FreezeControls(false);
		player VisionSetNakedForPlayer(getDvar("mapname"), 2.5);
		player Show();
	}
	else
	{
		wait 0.5;
	}

	// random primary weapon
	primaryWeapon = level.seekerWeapons[RandomInt(level.seekerWeapons.size)];
	player GiveWeapon(primaryWeapon + "_reflex_mp", RandomInt(9), false);
	player setSpawnWeapon(primaryWeapon + "_reflex_mp");
	player SwitchToWeapon(primaryWeapon + "_reflex_mp");

	// deagle + tactical knife
	player GiveWeapon("deserteagle_tactical_mp", 0, false);

	// single semtex
	player GiveWeapon("semtex_mp");

	// ammo
	player GiveMaxAmmo(primaryWeapon + "_reflex_mp");
	player GiveMaxAmmo("deserteagle_tactical_mp");

	// gametype-friendly perks
	player GivePerk("specialty_lightweight");
	player GivePerk("specialty_marathon");
	player GivePerk("specialty_falldamage");
	player GivePerk("specialty_fastreload");

	// Set player health high enough for stabbing to not instantly kill them
	player.maxhealth = 280;
	player.health = 280;

	/#
	player thread enableUnlimitedReload();
	player thread enableWeaponDicing("+actionslot 2");
	#/
	player thread giveHelpfulHintsForSeeker();
}

handleHider() {
	logPrint("handleHider called");

	player = self;

	player endon("disconnect");
	player endon("death");
	level endon("game_ended");

	// Wait for _playerlogic to give the player his default weapons and perks
	player waittill("spawned_player");

	// Go third-person mode
	player SetClientDvar("cg_thirdperson", 1);

	// Take weapons and perks
	player ClearPerks();
	player TakeAllWeapons();

	waittillframeend;

	player allowAds(false);
	player giveWeapon("deserteagle_tactical_mp", 0, false);
	player setWeaponAmmoClip("deserteagle_tactical_mp", 0);
	player setWeaponAmmoStock("deserteagle_tactical_mp", 0);
	player setSpawnWeapon("deserteagle_tactical_mp");
	player switchtoWeapon("deserteagle_tactical_mp");

	// Give gametype-friendly perks
	player GivePerk("specialty_quieter");
	player GivePerk("specialty_fastmantle");
	player GivePerk("specialty_falldamage");

	gameFlagWait("prematch_done");

	player thread enableThirdPersonSwitch("+actionslot 3", 1);
	player thread enableModelChange("+attack");
	player thread enableModelMenu("+smoke");
	player thread enableQuickRandomModelChange("+reload");
	player thread giveHelpfulHintsForHider();
	player thread revealWhenGameEnds();

	if (!gameFlag("seekers_released"))
	{
		gameFlagWait("seekers_prerelease_setup");

		rest = level.preReleaseTime - ((getTime() - level.preReleaseStart)/1000);
		player setLowerMessage("seekerreleasemsg", &"HIDEANDSEEK_TIMER_SEEKER_RELEASE", rest, 90);

		gameFlagWait("seekers_released");

		player clearLowerMessage("seekerreleasemsg");

		// "Seekers have been released." message on screen
		notifyData = spawnstruct();
		notifyData.titleText = &"HIDEANDSEEK_SEEKERS_RELEASED";
		notifyData.glowColor = (1.0, 0.4, 0.1);
		//notifyData.sound = game["seekers_released_sound"];
		player thread maps\mp\gametypes\_hud_message::notifyMessage(notifyData);
		player PlayLocalSound(game["seekers_released_sound"]);
	}

	// Downgrade from Marathon Pro to Marathon
	player UnSetPerk("specialty_fastmantle");
	player GivePerk("specialty_marathon");
}

setDisguiseModel(model)
{
	logPrint("setDisguiseModel called");

	player = self;

	displayName = maps\mp\mods\_modellist::getDisplayName(model);

	if (!IsDefined(player.disguiseEntity))
	{
		player Hide();
		player.disguiseEntity = Spawn("script_model", player.origin);
		//player.disguiseEntity LinkTo(player, "j_head", (0,0,0), (0,0,0));
		player.disguiseEntity.team = player.team;
		//player.disguiseEntity.pers["team"] = player.team;
		player.disguiseEntity.owner = player.owner;
		player.disguiseEntity Solid();
		player.disguiseEntity SetCanDamage(true);
		player thread watchDisguise("+melee");
		player thread watchDisguiseDamage();
	}
	player.disguiseEntity SetModel(model);
	player PingPlayer(); // flash on minimap for teammates
	if (IsDefined(game["hiders_changed_sound"]))
		player PlaySound(game["hiders_changed_sound"]);

	// Seems like changing model causes weapons to be reloaded
	player setWeaponAmmoClip("deserteagle_tactical_mp", 0);
	player setWeaponAmmoStock("deserteagle_tactical_mp", 0);

	player SetClientDvar("cg_thirdpersonrange", 300);
	player updateDisguiseView();

	//player SayTeam("(changed into " + displayName + ")");
	player IPrintLnBold("^2You changed into ^4" + displayName);
}

setRandomDisguiseModel()
{
	logPrint("setRandomDisguiseModel called");

	player = self;

	model = random(level.ModelList);
	if (!IsDefined(model))
		return;
	player setDisguiseModel(model.name);
}

deleteDisguiseModel()
{
	logPrint("deleteDisguiseModel called");

	player = self;

	if (!IsDefined(player.disguiseEntity))
		return;

	player.disguiseEntity Delete();
	if (IsDefined(game["hiders_changed_sound"]))
		player PlaySound(game["hiders_changed_sound"]);
	//player ShowViewModel();
	player Show();
}

updateDisguiseView()
{
	logPrint("updateDisguiseView called");

	player = self;

	// Do we even have an entity?
	if (!IsDefined(player.disguiseEntity))
		return;

	// Hide from third-person view of player
	if (player.isThirdPerson)
	{
		player.disguiseEntity ShowToPlayer(player);
	}
	else
	{
		player.disguiseEntity Hide();
		foreach (tplayer in level.players)
		{
			if (tplayer != player)
				player.disguiseEntity ShowToPlayer(tplayer);
		}
	}
}

/*
 * Binds a player command to let the player change between first
 * person view and third person view.
 */
enableThirdPersonSwitch(playerCommand, value) {
	logPrint("enableThirdPersonSwitch called");

	player = self;

	level endon("game_ended");
	player endon("death");
	player endon("disconnect");

	player.isThirdPerson = value;

	player notifyOnPlayerCommand("thirdpersonswitch_requested", playerCommand);

	for (;;)
	{
		player waittill("thirdpersonswitch_requested");

		// toggle third person mode
		value = !value;
		player SetClientDvar("cg_thirdperson", value);
		player.isThirdPerson = value;

		player updateDisguiseView();

		wait 0.5;
	}
}

/*
 * Binds a player command to let the player change into the model they
 * are looking at.
 */
enableModelChange(playerCommand) {
	logPrint("enableModelChange called");

	player = self;

	level endon("game_ended");
	player endon("death");
	player endon("disconnect");

	player notifyOnPlayerCommand("modelchange_requested", playerCommand);

	defaultOriginAdd = undefined;

	for (;;)
	{
		player waittill("modelchange_requested");

		if (!IsDefined(defaultOriginAdd))
		{
			defaultOriginAdd = (player GetEye()) - (player GetOrigin());
		}

		// Make a trace to the focused object
		origin = (player GetOrigin()) + defaultOriginAdd;
		ignore = player;
		if (IsDefined(player.disguiseEntity))
			ignore = player.disguiseEntity;
		trace = BulletTrace(origin, origin + anglestoforward(player getplayerangles()) * 1000000, false, ignore);
		if (!IsDefined(trace) || !IsDefined(trace["entity"]) || !IsDefined(trace["entity"].model) || !IsDefined(trace["entity"].classname))
		{
			player IPrintLnBold("^1Nothing to change into here.");
			continue;
		}
		if (trace["entity"].classname != "script_model")
		{
			player IPrintLnBold("^3" + trace["entity"].model + " is not a script model.");
			continue;
		}
		if (maps\mp\mods\_modellist::isIgnoredModel(trace["entity"].model))
		{
			player IPrintLnBold("^3You can't change into that!");
			continue;
		}

		player setDisguiseModel(trace["entity"].model);
	}
}

enableQuickRandomModelChange(playerCommand)
{
	LogPrint("enableQuickRandomModelChange called");

	player = self;

	player endon("disconnect");
	player endon("death");

	// Wait for player to request the quick random model change
	player notifyOnPlayerCommand("quickrandmodel_requested", playerCommand);
	for (;;)
	{
		player waittill("quickrandmodel_requested");
		player setRandomDisguiseModel();
	}
}

enableModelMenu(playerCommand)
{
	LogPrint("enableModelMenu called");

	player = self;

	player endon("disconnect");
	player endon("death");

	// Wait for player to try and open up the menu
	player notifyOnPlayerCommand("modelmenu_requested", playerCommand);
	player thread handleModelMenuSelection();
	for (;;)
	{
		player waittill("modelmenu_requested");
		player openPopupMenu(game["menu_models"]);
	}
}

handleModelMenuSelection()
{
	LogPrint("handleModelMenuSelection called");

	player = self;

	player endon("disconnect");
	player endon("death");

	for (;;)
	{
		player waittill("menuresponse", menu, response);

		// Make sure this is our menu that we got a response for
		if (!IsDefined(menu) || menu != "hns_models" || !IsDefined(response))
			continue;

		// Possible values: chmodel,<id>; back
		selection = StrTok(response, ",");
		switch (selection[0])
		{
			case "randmodel":
				player setRandomDisguiseModel();
				break;
			case "chmodel":
				if (selection.size != 2)
					break;
				modelid = int(selection[1]);
				if (modelid < 0)
					break;
				modelid = modelid % level.ModelList.size;
				player setDisguiseModel(level.ModelList[modelid].name);
				break;
		}
		player closePopupMenu();
	}
}

/*
 * Makes sure that the disguiseEntity attached to the "self" object
 * (a player) is always at the same position as the object itself.
 *
 * Also runs enableDisguiseModelAngleLocking() with the given player
 * command that allows to rotate the entity as intended.
 */
watchDisguise(angleLockingPlayerCommand)
{
	logPrint("watchDisguise called");

	player = self;

	level endon("game_ended");
	player endon("death");
	player endon("disconnect");

	player.disguiseAnglesLocked = false;
	player thread enableDisguiseModelAngleLocking(angleLockingPlayerCommand);

	for (;;)
	{
		wait 0.05;
		if (!IsDefined(player.disguiseEntity))
			break;
		player.disguiseEntity MoveTo(player GetOrigin(), 0.1);
		if (player.disguiseAnglesLocked)
			continue;
		player.disguiseEntity RotateTo(player GetPlayerAngles(), 0.1);
	}
}

watchDisguiseDamage()
{
	logPrint("watchDisguiseDamage called");

	player = self;

	level endon("game_ended");
	player endon("death");
	player endon("disconnect");

	for (;;)
	{
		player.disguiseEntity waittill("damage", amount, attacker, direction_vec, point, type, modelName, tagName, partName, iDFlags);
		attacker maps\mp\gametypes\_damagefeedback::updateDamageFeedback("standard");
		// eInflictor = the entity that causes the damage (e.g. a claymore)
		// eAttacker = the player that is attacking
		// iDamage = the amount of damage to do
		// sMeansOfDeath = string specifying the method of death (e.g. "MOD_PROJECTILE_SPLASH")
		// sWeapon = string specifying the weapon used (e.g. "claymore_mp")
		// damagepos = the position damage is coming from
		// damagedir = the direction damage is moving in
		player maps\mp\gametypes\_weapons::damageEnt(type, attacker, amount, "", modelName, point, direction_vec);
		//player.health = player.disguiseEntity.health;
	}

}

/*
 * Enables locking the angles of the disguise model using a player
 * command.
 */
enableDisguiseModelAngleLocking(playerCommand)
{
	logPrint("enableDisguiseModelAngleLocking called");

	player = self;

	level endon("game_ended");
	player endon("death");
	player endon("disconnect");

	player notifyOnPlayerCommand("disguisemodelanglelocking_requested", playerCommand);

	for (;;)
	{
		player waittill("disguisemodelanglelocking_requested");
		player.disguiseAnglesLocked = !player.disguiseAnglesLocked;
	}
}

/*
 * Binds a player command to shuffle the weapon.
 */
enableWeaponDicing(playerCommand)
{
	logPrint("enableWeaponDicing called");

	player = self;

	level endon("game_ended");
	player endon("death");
	player endon("disconnect");

	player notifyOnPlayerCommand("weapondicing_requested", playerCommand);

	for (;;)
	{
		player waittill("weapondicing_requested");

		// random primary weapon
		player TakeWeapon(player GetCurrentWeapon());
		primaryWeapon = random(level.seekerWeapons);
		player GiveWeapon(primaryWeapon + "_reflex_mp", RandomInt(9), false);
		player SwitchToWeapon(primaryWeapon + "_reflex_mp");
		player GiveMaxAmmo(primaryWeapon + "_reflex_mp");
	}
}

/*
 * Refills ammo stock after a reload.
 */
enableUnlimitedReload()
{
	logPrint("enableUnlimitedReload called");

	player = self;

	for (;;)
	{
		player waittill("reload");

		player GiveMaxAmmo(player GetCurrentWeapon());
	}
}

/*
 * Plays a revealing effect when the round ends if this player is
 * still alive.
 */
revealWhenGameEnds()
{
	logPrint("revealWhenGameEnds called");

	player = self;

	player endon("death");
	player endon("disconnect");

	level waittill("game_ended");

	for (a = 0; a < 5; a++) {
		fxPos = player GetOrigin() + (RandomIntRange(-32, 32), RandomIntRange(-32, 32), RandomIntRange(64, 192));
		PlayFX(game["fx_confetti"], fxPos);
		PlayFX(game["fx_boom"], fxPos);
		wait 0.75;
	}
}

/*
 * Shows gametype-specific hints for the hider in order to teach them how to play.
 */
giveHelpfulHintsForHider()
{
	player = self;

	player endon("death");
	player endon("disconnect");
	level endon("game_ended");

	gameFlagWait("prematch_done");

	player thread clearHintsShownOnDisconnect();

	player notifyOnPlayerCommand("hide_hint_for_disguise", "+attack");
	if (player showHintOnce("hider.disguise", &"HIDEANDSEEK_HINT_HIDER_DISGUISE"))
		player waittill("hide_hint_for_disguise");

	player notifyOnPlayerCommand("hide_hint_for_anglelock", "+melee");
	if (player showHintOnce("hider.anglelock", &"HIDEANDSEEK_HINT_HIDER_ANGLELOCK"))
		player waittill("hide_hint_for_anglelock");

	player notifyOnPlayerCommand("hide_hint_for_thirdperson", "+actionslot 3");
	if (player showHintOnce("hider.thirdperson", &"HIDEANDSEEK_HINT_HIDER_THIRDPERSON"))
		player waittill("hide_hint_for_thirdperson");

	player hideHint();
}

/*
 * Shows gametype-specific hints for the seeker in order to teach them how to play.
 */
giveHelpfulHintsForSeeker()
{
	player = self;

	player endon("death");
	player endon("disconnect");
	level endon("game_ended");

	gameFlagWait("prematch_done");
	gameFlagWait("seekers_released");

	player thread clearHintsShownOnDisconnect();

	player notifyOnPlayerCommand("hide_hint_for_semtex", "+frag");
	if (player showHintOnce("seeker.semtex", &"HIDEANDSEEK_HINT_SEEKER_SEMTEX"))
		player waittill("hide_hint_for_semtex");

	player hideHint();
}
